/*******************************************************************
*
* This file is part of the KDE project "Bovo"
*
* Bovo is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* Bovo is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with Bovo; see the file COPYING.  If not, write to
* the Free Software Foundation, 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301, USA.
*
********************************************************************/


/**
 * @file ai.cc implementing the Ai class
 */

#include "ai.h"

#include <QTimer>

#include "ai_impl.h"
#include "coord.h"
#include "dimension.h"
#include "move.h"

using namespace bovo;

/** namespace for AI stuff */
namespace ai {

Ai::Ai(const Dimension& dimension, Skill skill, Player player) {
    m_player = player;
    if (m_player == X) return;
    ai_init();
    table_size = dimension.width();
    setSkill(skill);
    ai_new_game();
}

Ai::~Ai() {
    ai_end();
}

/* public slots */

void Ai::changeBoard(const Move& move) {
    if (m_player == X) return;
    ai_step(move.x(), move.y());
}

void Ai::gameOver() {
}

void Ai::setSkill(Skill skill) {
    switch (skill) {
        case RidiculouslyEasy: start_depth = max_depth = 1; break;
        case VeryEasy: start_depth = max_depth = 2; break;
        case Easy: start_depth = max_depth = 3; break;
        case Normal: start_depth = max_depth = 4; break;
        case Hard: start_depth = max_depth = 5; break;
        case VeryHard: start_depth = max_depth = 6; break;
        case Impossible: start_depth = max_depth = 7; break;
    }
}

void Ai::slotMove() {
    Field f = ai_think();
    emit move(Move(m_player, Coord(f.x, f.y)));
}

} /* namespace ai */

#include "ai.moc"
