/***************************************************************************
 *   Copyright (C) 2007 by Pelladi Gabor   *
 *   pelladigabor@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#pragma once

#include <vector>
#include <string>

#include "ai_impl.h"

// a mark on the table
typedef unsigned char mark_T;
// occurrences of a certain position
typedef unsigned char count_T;
// small index for a constant array
typedef unsigned char index_T;

// table symbols indexed with player number
const mark_T mark[] = {'o', 'x', '!', '\0'};

// the game table
typedef mark_T table_T[max_table_size][max_table_size];
// the value of the fields for the two players for the next step
typedef count_T suggest_T[2][max_table_size][max_table_size];

// heuristic type
typedef signed short heur_T;

// maximal and minimal value of the heuristic function
const heur_T MaxHeur = 10000;
const heur_T MinHeur = -MaxHeur;
const heur_T HeurPercent = MaxHeur / 100;
const heur_T WinTreshold = 98 * HeurPercent;
// heuristic value decreases by this amount times the current depth
const heur_T depth_decay = 5;
// random seed to the heuristic value
const heur_T heur_seed = 2;

// number of categories used in the heuristic function
const index_T heurLevels = 6;
// occurrence of patterns for the two players
typedef count_T PatternCount[2][heurLevels];

// a row, column or diagonal of the table
typedef std::vector<mark_T> sample_T;
// interesting fields for the two players for the next step
typedef std::vector<Field> suggestions_T;

class Standing;
typedef void (Standing::*posf_T)(pos_T pos, int inv, count_T value0, count_T value1);

// game state class
class Standing {
public:
	table_T table;
	heur_T hval;
	bool target;
	index_T current;
	pos_T lastx, lasty;
	suggest_T suggest;

	// new game
	Standing();
	// copy a game state
	Standing(const Standing* other);

	// make a step
	void step(pos_T x, pos_T y);
	void step_server(pos_T x, pos_T y);

	// get the interesting fields for the two players for the next step
	void getSuggestions(suggestions_T& suggestions);
private:
	PatternCount matchCount;
	PatternCount matchCountRow[max_table_size];
	PatternCount matchCountColumn[max_table_size];
	PatternCount matchCountDiagonalSum[2 * max_table_size - 1];
	PatternCount matchCountDiagonalDiff[2 * max_table_size - 1];

	suggest_T suggestRow;
	suggest_T suggestColumn;
	suggest_T suggestDiagonalSum;
	suggest_T suggestDiagonalDiff;

	void posfRow(pos_T pos, int y, count_T value0, count_T value1);
	void posfColumn(pos_T pos, int x, count_T value0, count_T value1);
	void posfDiagonalSum(pos_T pos, int sum, count_T value0, count_T value1);
	void posfDiagonalDiff(pos_T pos, int diff, count_T value0, count_T value1);

	// calculate heuristic value
	void evaluate();

	// count occurrences of the patterns
	void countMatches();
	// this function consumes 95% of the thinking time
	void refresh(sample_T& sample, PatternCount& local, int inv, posf_T posf);
	// process pattern occurrences
	void decide();
};
